from django.urls import path

from . import views

app_name = 'outfit418backup'


urlpatterns = [
    path('', views.index, name='index'),
    path('dashboard/', views.dashboard, name='dashboard'),
    path('audit/', views.audit, name='audit'),
    path('find_jeremy/', views.find_jeremy, name='find_jeremy'),
]



from django.core.management.base import BaseCommand

from outfit418_backup_ratting.tasks import update_all_characters_logins


class Command(BaseCommand):
    help = 'Update login data for all characters'

    def handle(self, *args, **options):
        update_all_characters_logins.delay(True)
        self.stdout.write(self.style.SUCCESS('Tasks scheduled!'))



# Generated by Django 4.0.7 on 2022-09-12 08:41

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('eveonline', '0016_character_names_are_not_unique'),
        ('allianceauth_pve', '0012_alter_entry_options_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='ShareUser',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('character', models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, related_name='+', to='eveonline.evecharacter')),
                ('share', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='allianceauth_pve.entrycharacter')),
            ],
        ),
        migrations.CreateModel(
            name='EntryCreator',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('creator_character', models.ForeignKey(on_delete=django.db.models.deletion.RESTRICT, related_name='+', to='eveonline.evecharacter')),
                ('entry', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='allianceauth_pve.entry')),
            ],
        ),
    ]



from django.apps import AppConfig


class Outfit418BackupRattingConfig(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'outfit418_backup_ratting'

    def ready(self):
        import outfit418_backup_ratting.signals



from django.core.management.base import BaseCommand, CommandParser

from allianceauth.services.modules.teamspeak3.models import Teamspeak3User
from allianceauth.services.modules.teamspeak3.tasks import Teamspeak3Tasks


class Command(BaseCommand):
    help = 'Update ts groups for all users or a list of user ids'

    def add_arguments(self, parser: CommandParser):
        parser.add_argument('user_ids', nargs='*', type=int)

    def handle(self, *args, **options):
        if len(options['user_ids']) > 0:
            for pk in options['user_ids']:
                if Teamspeak3User.objects.filter(pk=pk).exists():
                    Teamspeak3Tasks.update_groups.delay(pk)
                    self.stdout.write(self.style.SUCCESS(f'Task scheduled for user with id {pk}'))
                else:
                    self.stdout.write(self.style.WARNING(f'User with id {pk} does not have a TS3 account!'))
        else:
            Teamspeak3Tasks.update_all_groups.delay()
            self.stdout.write(self.style.SUCCESS('Tasks scheduled for all users!'))



from django.contrib.auth.models import User

from esi.models import Token

from allianceauth.eveonline.models import EveCharacter
from allianceauth.authentication.models import CharacterOwnership

from .provider import esi


def get_or_create_char(character_id: int) -> EveCharacter:
    if character_id == 1:
        character_id = 2120413474

    char = EveCharacter.objects.get_character_by_id(character_id)

    if char is None:
        char = EveCharacter.objects.create_character(character_id)

    return char


def get_default_user() -> User:
    char = get_or_create_char(2120413474)
    return char.character_ownership.user


def get_user_or_fake(character_id) -> User:
    char = get_or_create_char(character_id)
    try:
        ownership = CharacterOwnership.objects.get(character=char)
        return ownership.user
    except CharacterOwnership.DoesNotExist:
        return get_default_user()


def get_ship_name(token: Token, item_ids: list[int]) -> list[str]:
    res = esi.client.Assets.post_characters_character_id_assets_names(
        character_id=token.character_id,
        item_ids=item_ids,
        token=token.valid_access_token()
    ).results()
    return [r['name'] for r in res]



from django import template

from allianceauth.eveonline.models import EveCharacter

register = template.Library()


@register.filter
def ct_is_active(char: EveCharacter) -> bool:
    return char.characteraudit.is_active() if hasattr(char, 'characteraudit') else False



from celery import shared_task, group
from celery_once import QueueOnce

from django.db import transaction
from django.utils import timezone

from allianceauth.services.hooks import get_extension_logger
from allianceauth.authentication.models import CharacterOwnership

from allianceauth_pve.models import Rotation, Entry, EntryCharacter

from corptools.models import CharacterAudit
from corptools.task_helpers.char_tasks import get_token

from .utils import get_or_create_char, get_user_or_fake, get_default_user
from .models import EntryCreator, ShareUser, CharacterAuditLoginData
from .provider import esi


logger = get_extension_logger(__name__)


@shared_task
def fetch_char(char_id):
    if char_id != 1:
        get_or_create_char(char_id)


@shared_task
@transaction.atomic
def save_import(data):
    fake_user = get_default_user()

    for rotation_data in data:
        rotation = Rotation.objects.create(
            name=rotation_data['name'],
            actual_total=int(rotation_data['actual_total']),
            tax_rate=rotation_data['tax_rate'],
            is_closed=rotation_data['is_closed'],
            is_paid_out=rotation_data['is_paid_out'],
            priority=rotation_data['priority'],
        )

        if rotation_data['name'] == '':
            rotation.name = f"Rotation {rotation.pk}"
            rotation.save()

        Rotation.objects.filter(pk=rotation.pk).update(
            created_at=rotation_data['created_at'],
            closed_at=rotation_data['closed_at'],
        )

        for entry_data in rotation_data['entries']:

            if len(entry_data['shares']) > 0:
                creator = get_user_or_fake(entry_data['created_by'])
                char = get_or_create_char(entry_data['created_by'])

                entry = rotation.entries.create(
                    estimated_total=entry_data['estimated_total'],
                    created_by=creator,
                )

                if creator == fake_user:
                    EntryCreator.objects.create(
                        entry=entry,
                        creator_character=char
                    )

                Entry.objects.filter(pk=entry.pk).update(
                    created_at=entry_data['created_at'],
                    updated_at=entry_data['updated_at'],
                )

                role = entry.roles.create(
                    name='Krab',
                    value=1,
                )

                for share_data in entry_data['shares']:
                    user = get_user_or_fake(share_data['character'])
                    character = get_or_create_char(share_data['character'])

                    share = entry.ratting_shares.create(
                        user=user,
                        user_character=character,
                        role=role,
                        site_count=share_data['share_count'],
                        helped_setup=share_data['helped_setup'],
                    )

                    if user == fake_user:
                        ShareUser.objects.create(
                            share=share,
                            character=character
                        )
        if rotation.entries.count() == 0:
            rotation.delete()


@shared_task
def update_fake_users():
    characters = ShareUser.objects.all().values('character')

    for ownership in CharacterOwnership.objects.filter(character__in=characters):
        with transaction.atomic():
            shares_qs = ShareUser.objects.filter(character=ownership.character)
            EntryCharacter.objects.filter(pk__in=shares_qs.values('share_id')).update(user=ownership.user)
            shares_qs.delete()

    characters = EntryCreator.objects.all().values('creator_character')

    for ownership in CharacterOwnership.objects.filter(character__in=characters):
        with transaction.atomic():
            entry_qs = EntryCreator.objects.filter(creator_character=ownership.character)
            Entry.objects.filter(pk__in=entry_qs.values('entry_id')).update(created_by=ownership.user)
            entry_qs.delete()


@shared_task(base=QueueOnce, once={'keys': ['pk'], 'graceful': True})
def update_character_login(pk, force_refresh=False):
    char = CharacterAudit.objects.get(pk=pk)
    login_data = CharacterAuditLoginData.objects.get_or_create(characteraudit=char)[0]

    if force_refresh or login_data.last_update is None or login_data.last_update < timezone.now() - timezone.timedelta(hours=1):
        token = get_token(char.character.character_id, ['esi-location.read_online.v1'])
        if token:
            result = (
                esi.client
                .Location
                .get_characters_character_id_online(
                    character_id=char.character.character_id,
                    token=token.valid_access_token()
                )
                .results()
            )
            if result['online']:
                login_data.last_login = timezone.now()
                login_data.last_update = timezone.now()
            elif 'last_login' in result:
                login_data.last_login = result['last_login']
                login_data.last_update = timezone.now()
            login_data.save()


@shared_task
def update_all_characters_logins(force_refresh=False):
    pks = CharacterAudit.objects.values_list('pk', flat=True)
    group(update_character_login.s(pk=pk) for pk in pks).delay(force_refresh=force_refresh)



# Generated by Django 4.2.11 on 2024-05-05 13:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('corptools', '0102_corporationaudit_last_change_contracts_and_more'),
        ('outfit418_backup_ratting', '0002_general'),
    ]

    operations = [
        migrations.CreateModel(
            name='CharacterAuditLoginData',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('last_login', models.DateTimeField(blank=True, null=True)),
                ('last_update', models.DateTimeField(blank=True, null=True)),
                ('characteraudit', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='corptools.characteraudit')),
            ],
        ),
    ]



from django.db.models.signals import post_save
from django.dispatch import receiver

from corptools.models import CharacterAudit

from .tasks import update_character_login


@receiver(post_save, sender=CharacterAudit)
def update_login_data(sender, instance, **kwargs):
    update_character_login.apply_async(kwargs={'pk': instance.pk, 'force_refresh': False}, countdown=10)



from django.db import models

from allianceauth_pve.models import Entry, EntryCharacter

from allianceauth.eveonline.models import EveCharacter

from corptools.models import CharacterAudit


class General(models.Model):
    class Meta:
        managed = False
        default_permissions = ()
        permissions = (
            ('audit_corp', "Can audit corp members' alts"),
            ('find_jeremy', "Can find Jeremy"),
        )


class EntryCreator(models.Model):
    entry = models.OneToOneField(Entry, on_delete=models.CASCADE, related_name='+')
    creator_character = models.ForeignKey(EveCharacter, on_delete=models.RESTRICT, related_name='+')


class ShareUser(models.Model):
    share = models.OneToOneField(EntryCharacter, on_delete=models.CASCADE, related_name='+')
    character = models.ForeignKey(EveCharacter, on_delete=models.RESTRICT, related_name='+')


class CharacterAuditLoginData(models.Model):
    characteraudit = models.OneToOneField(CharacterAudit, on_delete=models.CASCADE, related_name='+')
    last_login = models.DateTimeField(null=True, blank=True)
    last_update = models.DateTimeField(null=True, blank=True)



from allianceauth import hooks
from allianceauth.services.hooks import UrlHook, MenuItemHook
from allianceauth.services.hooks import get_extension_logger

from . import urls

logger = get_extension_logger(__name__)


class Outfit418BackupRattingItemHook(MenuItemHook):
    def __init__(self):
        super().__init__("Outfit418 Backup Ratting", "fas fa-wallet", "outfit418backup:index", navactive=['outfit418backup:index', 'outfit418backup:dashboard'])

    def render(self, request):
        if request.user.is_superuser:
            return super().render(request)
        return ''


class Outfit418AuditHook(MenuItemHook):
    def __init__(self):
        super().__init__("418 Audit", "fas fa-search", "outfit418backup:audit", navactive=['outfit418backup:audit'])

    def render(self, request):
        if request.user.has_perm('outfit418_backup_ratting.audit_corp'):
            return super().render(request)
        return ''


class Outfit418FindJeremyHook(MenuItemHook):
    def __init__(self):
        super().__init__("Find Jeremy", "fas fa-search", "outfit418backup:find_jeremy", navactive=['outfit418backup:find_jeremy'])

    def render(self, request):
        if request.user.has_perm('outfit418_backup_ratting.find_jeremy'):
            return super().render(request)
        return ''


@hooks.register('menu_item_hook')
def register_menu():
    return Outfit418BackupRattingItemHook()


@hooks.register('menu_item_hook')
def register_audit_menu():
    return Outfit418AuditHook()


@hooks.register('menu_item_hook')
def register_find_jeremy_menu():
    return Outfit418FindJeremyHook()


@hooks.register('url_hook')
def register_urls():
    return UrlHook(urls, 'outfit418backup', 'outfit418backup/')



import pickle
from collections import defaultdict

from django.contrib import messages
from django.contrib.auth.decorators import user_passes_test, login_required, permission_required
from django.shortcuts import render, redirect
from django.db.models import F, Prefetch, Max, Subquery, OuterRef, Case, When, Min, Exists
from django.db.models.lookups import LessThan
from django.utils import timezone

from celery import group

from allianceauth.services.hooks import get_extension_logger
from allianceauth.authentication.models import CharacterOwnership
from allianceauth.eveonline.models import EveCharacter

from corptools.models import CharacterAudit, CharacterAsset
from corptools.task_helpers.char_tasks import get_token

from .forms import BackupForm
from .tasks import save_import, fetch_char
from .models import CharacterAuditLoginData
from .utils import get_ship_name

logger = get_extension_logger(__name__)


@login_required
@user_passes_test(lambda user: user.is_superuser)
def index(request):
    return redirect('outfit418backup:dashboard')


@login_required
@user_passes_test(lambda user: user.is_superuser)
def dashboard(request):
    if request.method == 'POST':
        form = BackupForm(request.POST, request.FILES)
        if form.is_valid():
            data = pickle.load(form.cleaned_data['file'])

            group((fetch_char.si(char_id) for char_id in data['character_list'])).delay()
            save_import.apply_async(kwargs={'data': data['rotations']}, countdown=30)
            messages.success(request, 'Backup task will start in 30 seconds!')
            return redirect('allianceauth_pve:index')
        else:
            messages.error(request, 'Form not valid!')
    else:
        form = BackupForm()
    context = {
        'form': form
    }
    return render(request, 'outfit418_backup_ratting/index.html', context=context)


@login_required
@permission_required('outfit418_backup_ratting.audit_corp')
def audit(request):
    corp_id = request.user.profile.main_character.corporation_id
    ownership_qs = (
        CharacterOwnership.objects
        .select_related('character__characteraudit')
        .annotate(
            last_login=Subquery(
                CharacterAuditLoginData.objects
                .filter(characteraudit__character=OuterRef('character'))
                .values('last_login')
            )
        )
    )
    user_login_qs = (
        CharacterAuditLoginData.objects
        .filter(
            characteraudit__character__character_ownership__user=OuterRef('character__character_ownership__user')
        )
        .values('characteraudit__character__character_ownership__user')
    )

    mains = (
        CharacterAudit.objects
        .filter(
            character__character_ownership__user__profile__main_character=F('character'),
            character__corporation_id=corp_id,
        )
        .select_related('character__character_ownership__user')
        .prefetch_related(
            Prefetch(
                'character__character_ownership__user__character_ownerships',
                queryset=ownership_qs,
                to_attr='chars',
            ),
        )
        .annotate(
            last_login=Subquery(
                user_login_qs
                .annotate(last_login=Max('last_login'))
                .values('last_login')
            )
        )
        .annotate(
            is_updating=Case(
                When(
                    LessThan(
                        Subquery(
                            user_login_qs
                            .annotate(last_update=Min('last_update'))
                            .values('last_update')
                        ),
                        timezone.now() - timezone.timedelta(days=1),
                    ) |
                    Exists(
                        CharacterAuditLoginData.objects
                        .filter(
                            characteraudit__character__character_ownership__user=OuterRef('character__character_ownership__user'),
                            last_update__isnull=True
                        )
                    ),
                    then=False
                ),
                default=True,
            )
        )
        .annotate(
            older_last_update=Case(
                When(
                    Exists(
                        CharacterAuditLoginData.objects
                        .filter(
                            characteraudit__character__character_ownership__user=OuterRef('character__character_ownership__user'),
                            last_update__isnull=True
                        )
                    ),
                    then=None
                ),
                default=Subquery(
                    user_login_qs
                    .annotate(last_update=Min('last_update'))
                    .values('last_update')
                )
            )
        )
    )

    return render(request, 'outfit418_backup_ratting/audit.html', context={'mains': mains})


@login_required
@permission_required('outfit418_backup_ratting.find_jeremy')
def find_jeremy(request):
    thannys = CharacterAsset.objects.filter(type_name_id=23911).select_related('character__character')
    thanny_dict = defaultdict(list)
    jeremy_owners = defaultdict(list)

    for thanny in thannys:
        thanny_dict[thanny.character.character].append(thanny.item_id)

    for char, item_ids in thanny_dict.items():
        token = get_token(char.character_id, ['esi-assets.read_assets.v1'])
        if token:
            names = get_ship_name(token, item_ids)
            for name in names:
                if 'jeremy' in name.lower():
                    jeremy_owners[char].append(name)
                    break

    context = {
        'jeremy_owners': jeremy_owners,
    }

    return render(request, 'outfit418_backup_ratting/find_jeremy.html', context)



# Generated by Django 4.2.16 on 2024-11-21 17:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('outfit418_backup_ratting', '0003_characterauditlogindata'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='general',
            options={'default_permissions': (), 'managed': False, 'permissions': (('audit_corp', "Can audit corp members' alts"), ('find_jeremy', 'Can find Jeremy'))},
        ),
    ]



# Generated by Django 4.2.8 on 2024-01-04 16:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('outfit418_backup_ratting', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='General',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
            options={
                'permissions': (('audit_corp', "Can audit corp members' alts"),),
                'managed': False,
                'default_permissions': (),
            },
        ),
    ]
